/* -*- c-basic-offset: 4 indent-tabs-mode: nil -*-  vi:set ts=8 sts=4 sw=4: */
/*
Copyright (C) 2001 Paul Davis 
Copyright (C) 2004-2006 Grame

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "JackLinuxMemoryLock.h"

#include "JackError.h"

#include <limits.h>
#include <unistd.h>
#include <sys/mman.h>

#define UNLOCK_LARGE_LIBRARIES 1

namespace Jack
{

int JackLinuxMemoryLock::LockMemory(const void *address, uint32_t size)
{
    if (mlock(address, size) != 0) {
        jack_error("Cannot lock down memory area (%s)", strerror(errno));
        return -1;
    } else {
        fprintf(stderr, "succeeded in locking %u-byte memory area\n", size);
    }
}
	
int JackLinuxMemoryLock::LockJackMemory()
{
    // cc: scribble on the stack to ensure some of it is mapped:
    // borrowed from jackd.  I don't think this is actually necessary,
    // but I don't think it does any harm.
    char buf[100000];
    for (int i = 0; i < 100000; i++) buf[i] = (char) (i & 0xff);

    if (mlockall(MCL_CURRENT | MCL_FUTURE) != 0) {
	jack_error("Cannot lock down memory for RT thread (%s)", strerror(errno));
	return -1;
    } else {
	fprintf(stderr, "succeeded in locking memory\n");
    }
    
#ifdef UNLOCK_LARGE_LIBRARIES

    static char* blacklist[] = {
	"/libgtk",
	"/libgdk",
	"/libqt",
	"/libQt",
	"/libkde",
	"/libX",
	"/libfreetype",
	"/libfltk",
	"/libpango",
	"/libssl",
	"/plugins/styles/",
	"/plugins/imageformats/",
	"/fonts/",
	"/engines/",
	"/wine/",
	NULL
    };

    static char* whitelist[] = {
	"/libc-",
        "/libQtCore",
	NULL
    };

    //!!! if we're using substrings, we only need /lib/ as all the
    // others contain it!  Is there any real risk of false positives?
    static char* library_substrings[] = {
	"/lib/",
	"/usr/lib/",
	"/usr/local/lib/",
	"/usr/X11R6/lib/",
	"/opt/lib/",       /* solaris-y */   
	"/opt/local/lib/", /* common on OS X */
	NULL
    };

    FILE* map;
    size_t start;
    size_t end;
    char path[PATH_MAX+1];
    int unlock;
    int i;
    int whoknows;
    int looks_like_library;

    snprintf (path, sizeof(path), "/proc/%d/maps", getpid());

    if ((map = fopen (path, "r")) == NULL) {
	fprintf (stderr, "can't open map file\n");
	return 0;
    }

    while (!feof (map)) {

	unlock = 0;

	if (fscanf (map, "%x-%x %*s %*x %*d:%*d %d",
		    &start, &end, &whoknows) != 3) {
	    break;
	}

	/* cc -- get this read before checking the last number,
	   as some entries may have a zero followed by e.g. "[heap]" */
	fscanf (map, " %[^\n]", path);

	if (!whoknows) {
	    continue;
	}

	/* if it doesn't look like a library, forget it */

	looks_like_library = 0;

	for (i = 0; library_substrings[i]; ++i) {
	    if ((looks_like_library = (strstr (path, library_substrings[i]) != NULL))) {
		break;
	    }
	}

	if (!looks_like_library) {
	    continue;
	}
		
	for (i = 0; blacklist[i]; ++i) {
	    if (strstr (path, blacklist[i])) {
		unlock = 1;
		break;
	    }
	}

	if (end - start > 1048576) {
	    unlock = 1;
	}
		
	for (i = 0; whitelist[i]; ++i) {
	    if (strstr (path, whitelist[i])) {
		unlock = 0;
		break;
	    }
	}
		
	if (unlock) {
	    fprintf (stderr, "unlocking %s (%d bytes)\n", path, (int)(end - start));
	    munlock ((char *) start, end - start);
	}
    }

    fclose (map);

#endif
    
    return 0;
}

} // end of namespace


